classdef dataClass
%DATACLASS class used to store and to modify data points.
% 
% The data is stored in three matrices. RawData takes all 
% datapoints and consists of three columns:
%       [globalcount, xvalue, yvalue, selectionflag ]
% selected is the selected subset of rawData and for
% averagedData the averaging is done by the xvalue
% 
% You can add datapoints with functions
%    addDataPoint(globalcount,xvalue,yvalue)
% or by calling with a paramterset
%    addDataParameter()
%
% plot needs an axes to show the data
%
% Elmar, 2.2.06


properties 
    
    %***************************
    % data representation
    %***************************
    rawData      = [NaN,NaN,NaN,NaN];  % [globalcount, xvalue, yvalue, selectionflag ]
    selectedData = [NaN,NaN,NaN];      % [globalcount, xvalue, yvalue] without removed points
    averagedData = [NaN,NaN,NaN];      % xvalues, ymean, ystd
    numRawPoints = 0;                  % total number of data points
    numSelPoints = 0;                  % total number of selected points

    %***************************
    % names of the axes
    %***************************
    xName = '...';
    yName = '...'; 
    setName = '';


    %***************************
    % flags, what to to the the 
    % data
    %***************************
    showSinglePoints  = false;
    showDataPoints    = true;
    showRemovedPoints = false;
    showErrorbars     = true;
    showErrorPatch    = false;
    doAverage         = false;
    doUpdate          = false;


    %***************************
    % plots
    %***************************
    plotColor         = [0, 0, 1];
    markerString      = {'o','s','<','v',};
    markerSelection   = 1;
    markerSize        = 8;

    
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% public functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

methods 
       
    %***************************
    % Constructor
    %***************************
    function obj = dataClass(xparam,yparam)
        obj.xName = xparam;
        obj.yName = yparam;
    end

    %*********************************
    % set data names
    %*********************************
    function obj = setNames(obj,xparam,yparam)
        obj.xName = xparam;
        obj.yName = yparam;
    end    


    %*********************************
    % remove point in interval
    %*********************************
    function obj = removePoints(obj,xmin,xmax,ymin,ymax)
        if obj.numSelPoints>0
            [ind,v] = find( obj.selectedData(:,2)>=xmin & obj.selectedData(:,2)<=xmax & ...
                        obj.selectedData(:,3)>=ymin & obj.selectedData(:,3)<=ymax );
            obj.rawData(obj.findRawIndex(ind),4) = 0;
            obj.selectedData(ind,:) = [];
            obj.numSelPoints = size(obj.selectedData,1);
            obj = obj.updateData;
        end
    end


    %*********************************
    % reset object
    %*********************************
    function obj = reset(obj)
        obj.rawData       = [NaN,NaN,NaN,NaN]; 
        obj.selectedData  = [NaN,NaN,NaN]; 
        obj.averagedData  = [NaN,NaN,NaN];
        obj.numRawPoints  = 0;
        obj.numSelPoints  = 0;
    end


    %*****************************
    % is called after gui changes
    %*****************************
    function obj = updateData(obj)
        if obj.doAverage
            obj.averagedData = mgroup([ obj.selectedData(:,2), obj.selectedData(:,3),  obj.selectedData(:,3) ],['mean';'stdd']);
        end
    end


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Private functions
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    function rawIndex = findRawIndex(obj,selIndex)
        rawIndex = find( ismember(obj.rawData(:,1),obj.selectedData(selIndex,1)) );
    end

    %*********************************
    % remove selected points
    %*********************************
    function obj = reselectData(obj)
        ind = find(obj.rawData(:,4) == 1 );
        obj.selectedData = [ obj.rawData(ind,1), obj.rawData(ind,2), obj.rawData(ind,3)];
        obj.numSelPoints = size(obj.selectedData,1);
        obj.numRawPoints = size(obj.rawData,1);
        if obj.doAverage
           obj.averagedData = mgroup([ obj.selectedData(:,2), obj.selectedData(:,3),  obj.selectedData(:,3) ],['mean';'stdd']);
        end
    end


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % import and expoert functions
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function saveData(obj,fname)
        % save mat file        
        try 
            filename = [fname,'_obj.mat'];            
            save(filename,'obj');
        catch
            disp(['Could not save object file ',fname,'.']);
            disp(lasterr);
        end
        
        % save ascii file
        try
            filename = [fname,'_raw.txt'];
            raw      = obj.rawData;
            fid      = fopen(filename, 'wt');
            fprintf(fid, '%s\n%s\n', obj.xName,obj.yName);
            fclose(fid)
            save(filename,'raw','-ascii','-double','-tabs','-append');
        catch
            disp(['Could not save text file ',fname,'.']);
            disp(lasterr);
        end
    end

    % load 2-ascii or 1-object
    function obj = loadData(obj,fname,mode)
        if ~exist(fname,'file')
            disp(['File ',fname',' does not exist']);
            return;
        end
        switch mode
            case 1
                try
                    obj = load(fname);
                catch
                    disp(['Could not load file ',fname]);
                    disp(lasterr);
                end
            case 2
                try
                    fid = fopen(fname, 'r');
                    obj.xName = fgetl(fid);
                    obj.yName = fgetl(fid);
                    C = textscan(fid, '%n %n %n %n');
                    obj.rawData = [];
                    obj.rawData(:,1) = C{1};
                    obj.rawData(:,2) = C{2};
                    obj.rawData(:,3) = C{3};
                    obj.rawData(:,4) = C{4};
                    fclose(fid);
                    obj.numRawPoints = size(C,2);
                    obj = obj.reselectData;
                catch
                    disp(['Could not load file ',fname]);
                    disp(lasterr);
                end
        end
    end

    function obj = editObject(obj)
        obj = gui_dataset(obj);
    end
    
end
end
