classdef fitClass
%FITCLASS combines fitparameters and fitting
% functions. This is the template class
% for specific fitting classes like 
% poly1 or exponential...
%
% This class is uses a linear fit as an 
% example. Do overload functions
% 
% - startPrePoints(obj, haxes)
% - pre2StartPoints(obj)
% - createOutput(obj)
% 
% and variables:
%
% fitName
% fitNumber
% fType
% fOpts
% 
% Elmar, 25.3.2007


properties


    %***************************
    % fit properties
    %***************************
    doFit     = false;     % is it active?
    showFit   = true;      % plot fit
    fResult   = [];        % fit result object
    gof       = [];
    fType     = fittype;   % fit type
    fOpts     = {};        % options object
    fitNumber = 1;
    fitName   = 'linear';
    minPoints = 3;         % minimum number of datapoints
    dataSet   = [true true true true];
    param     = [0 0 0];


    %**************************
    % presentation
    %**************************
    plotColor  = [0,0,0];
    handle     = [];


    %**************************
    % output Strings
    %**************************
    smallStr = '';
    bigStr   = {''};
    tabEntries = {};

    %**************************
    % preselection
    %**************************
    doPreSetting = true;
    prePoints    = [];
    adoptPoints  = false;

end


methods

% <--------------------start:  edit for new fittpye  -------------------->



%*********************************
% Init fitting stuff
% initiate what ever is 
% needed
%*********************************
function obj = initFitting(obj)
    obj.fType     = fittype('poly1');   
    obj.fOpts{1}  = fitoptions('Method','NonlinearLeastSquares');
    obj.minPoints = 3;  
    obj.fitName   = 'linear';
end


%*********************************
% init prepoints
% calculate the prepoints
% for the first time
% (just from min/max of axes)
%*********************************
function obj = startPrePoints(obj, haxes)
    dx = xlim(haxes);
    dy = ylim(haxes);
    x  = [dx(1) dx(1)] + abs(dx(2)-dx(1))*[0.1 0.9];
    y  = [dy(1) dy(1)] + abs(dy(2)-dy(1))*[0.1 0.9];
    obj.prePoints(1,:) = [x(1) y(1)];
    obj.prePoints(2,:) = [x(2) y(2)];
end

%*********************************
% calc the start parameters for 
% fit. Either from prePoints
% or take them from the last fit.
%*********************************
function obj = getStartPoints(obj,haxes)
    
    if obj.adoptPoints && ~isempty(obj.fResult)
        obj.fOpts{1}.StartPoint = [obj.fResult.p1 obj.fResult.p2];
    else
        % start points [p1 p2], from prepoints
         if isempty(obj.prePoints)
              obj = obj.startPrePoints(haxes);
         end
         p1 = (obj.prePoints(2,2)-obj.prePoints(2,1))/(obj.prePoints(1,2)-obj.prePoints(1,1));
         p2 = p1 * obj.prePoints(1,1) + obj.prePoints(1,2);
         obj.fOpts{1}.StartPoint = [p1 p2];
    end
end

%******************************************
% calc fit function for given
% prePoints (for dashed line on plot)
%******************************************
function [xrange,yrange] = calcDataPoints(obj,xrange,prePoints)
        p1 = (prePoints(2,2)-prePoints(1,2))/(prePoints(2,1)-prePoints(1,1));
        p2 = prePoints(1,2) - p1 * prePoints(1,1);
        yrange = p1*xrange + p2;
end

%******************************************
% calc prePoints from old fit results
% (to set new prePoints)
%******************************************
function obj = last2PrePoints(obj, haxes)
    if isempty(obj.fResult)
        obj = obj.startPrePoints(haxes);
        return;
    end
    dx = xlim(haxes);
    dy = ylim(haxes);
    x  = [dx(1) dx(1)] + abs(dx(2)-dx(1))*[0.1 0.9];
    y =  obj.fResult.p1 .* x + obj.fResult.p2;
    obj.prePoints(:,1) = x;
    obj.prePoints(:,2) = y;
end


%***************************************
% write output to strings
% bigStr - is fit function given in 
% extra widget
% smallStr - is string in gui
%***************************************
function obj = createOutput(obj)
    fCoeffs = coeffvalues(obj.fResult);
    fSigma  = confint(obj.fResult);
    deltaP1 = max( abs(fSigma(:,1)-[obj.fResult.p1; obj.fResult.p1]) );
    deltaP2 = max( abs(fSigma(:,2)-[obj.fResult.p2; obj.fResult.p2]) );
    
    obj.bigStr = {'\bfModel: f(x) = a + b x',...
                  '\rmno parameters unused'};
    obj.smallStr = sprintf('%-2.2e',obj.fResult.p1);
    obj.tabEntries = {'a', num2str(obj.fResult.p2), num2str(deltaP2); ...
                      'b', num2str(obj.fResult.p1), num2str(deltaP1); ...
                      '','','';...
                      'xi2',num2str(obj.gof.rsquare),''};
end



% < --------------stop, edit for new fittpye ------------------->




%***************************
% Constructor
%***************************
function obj = fitClass
    obj.param = [0,0,0];
    obj = obj.initFitting();
end




%****************************************************************
% Fitting 
%****************************************************************


%***************************
% runFit
%***************************
function obj = runFit(obj, seqObj, haxes)

    %*************
    % get data
    %*************
    xdata = [];
    ydata = [];
	for i=1:seqObj.numData
        if obj.dataSet(i) && seqObj.listData{i}.numSelPoints > 0
            xdata = [xdata; seqObj.listData{i}.selectedData(:,2)];
            ydata = [ydata; seqObj.listData{i}.selectedData(:,3)];
        end
    end

    %*********************
    % check datasize
    %*********************
    if size(xdata,1)<obj.minPoints
        disp('Datasize too small.');
        return;
    end
    % kill NaN
    ind = find(isnan(xdata));
    xdata(ind) = [];ydata(ind) = [];
    ind = find(isnan(ydata));
    xdata(ind) = [];ydata(ind) = [];
    
    

    %*********************
    % do fitting
    %*********************
    if obj.doPreSetting
        obj = obj.getStartPoints(haxes);
    end
    
    try 
        [obj.fResult,obj.gof] = fit( xdata, ydata, obj.fType, obj.fOpts{1});
        obj = obj.createOutput;
    catch
        disp(['Problems during fiting']);
        disp(lasterr);
        disp('');
    end

end



%***************************
% plot fit
%***************************
function obj = plotFit(obj,haxes)
    if ~isempty( obj.fResult )
        dx = xlim(haxes);
        xrange = linspace(dx(1),dx(2),300);
        hold(haxes,'on');
        obj.handle = plot(haxes,xrange,obj.fResult(xrange), ...
                         'color',obj.plotColor, ...
                         'linewidth',1.5);
        hold(haxes,'off');                     
    end
end


%***************************
% clear fit
%***************************
function obj = clearFit(obj,haxes)
    obj.smallStr   = '';
    obj.bigStr     = {''};
    obj.fResult    = [];
    obj.gof        = [];
    obj.prePoints  = [];
    obj.tabEntries = {};
    if ishandle(obj.handle)
        delete(obj.handle)
    end
    obj = obj.startPrePoints(haxes);
end


end


end
