classdef paramClass
%PARAMCLASS storage class for parameters
% Stores control parameters and calculated values. 
% There are a couple of arrays and cells:  
%    o titles, cell with the names of the blocks
%    o names,  cell with actual parameter names
%    o tindex, array with title index of parameters
%    o types,  cell with type names
%    o values, array with values of parameters
%    o svalues, cell with strings of previous values
%    o changed, array with flags for every parameter 
%               whether it has changed
% 
% SYNTAX 
%       constructor       - call with filename or without
%       loadParameterFile - intern, load parameters from textfile
%       addParameter      - extern, add parameter to list
%       getValue          - extern, get value and string for parameter
%       checkChange       - compare with other paramObj if values have
%                           changed
%
% Elmar, Innsbruck 12.08.06


properties
    
        
%**************************************
% representation
%**************************************
titles@cell   = {}; 
names@cell    = {}; 
tindex@double = [];
types@cell    = {};
values@cell   = {};
svalues@cell  = {};
changed@logical = true;

end

methods


%**************************************
% Constructor
%**************************************
function obj = paramClass(fileName)
%    construct obj;
    if nargin == 1
        obj = obj.loadParameterFile(fileName);
    end
end



%**************************************
% loadParameterFile
%**************************************
function obj = loadParameterFile(obj, fileName)
    % load from prameters from file
    % read textfile line by line
    % and take Blocks with tilte and flag H
    if ~exist(fileName,'file')
        disp(['Could not open file :',fileName]);
        return;
    end
    
    fID = fopen(fileName);
    takeBlock = true;
    inLine    = fgetl( fID );
    
    while ischar(inLine)

        % take string apart
        pos      = regexp(inLine, '=' );
        prefix   = inLine(1);
        name     = inLine(2:pos-1);
        instring = inLine(pos+1:end);

        switch prefix
            case 'T'
                title = inLine(2+4:end-4);
            case 'H'
                value = strcmp('TRUE',instring);
                if value
                    takeBlock = true;
                else
                    takeBlock = false;
                end
            case 'B'
                if takeBlock
                    obj = obj.addParameter(title,name,'boolean',strcmp('TRUE',instring),instring);
                end
            case 'D'
                if takeBlock
                    obj = obj.addParameter(title,name,'double',str2double( instring ),instring);
                end
            case 'I'
                if takeBlock
                    obj = obj.addParameter(title,name,'integer',str2num( instring ),instring);
                end
            case 'S'
                if takeBlock
                    obj = obj.addParameter(title,name,'string', instring, instring);
                end
            otherwise
                disp(['Could not identify line:' inLine,'.']);
        end
        inLine = fgetl( fID );
    end

    fclose( fID );
end




%**************************************
% addParameter
%**************************************
function obj = addParameter(obj,title,name,type,value,string)
    % add a single parameter with title, name ...

    % check arguments
    if sum(~ischar([title,name,type,string]))
        error('Wrong entry for addParameter: title, name, type, value, string.');
        return;
    end;
    
    % check for title 
    t = find(strcmp( obj.titles, title));
    if isempty(t)
        t =  size(obj.titles,2)+1;
        obj.titles{t} = title;
    end
    
    % add to list
    ll = size(obj.values,2) + 1;
    obj.names{ll}   = name;
    obj.tindex(ll)  = t;
    obj.types{ll}   = type;
    obj.values{ll}  = value;
    obj.svalues{ll} = string;
    obj.changed(ll) = true;
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% getValue
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [value,str] = getValue(obj,name)
    % get the value and string of 
    % parameter with name 'name'
    
    ind   = find( strcmp(obj.names,name) );
    if isempty(ind)
        value = [];
        str = [];
    else
        value = obj.values{ind(end)};
        str   = obj.svalues{ind(end)};
    end
end


function [value,str] = getTValue(obj,title, name)
    % get the value and string of 
    % parameter with name 'name' and title 'title'
    
    value = []; str = [];               
    % search title
    t_ind = find( strcmp( obj.titles, title) );
    if isempty(t_ind)
        return;
    end
    % search list
    l_ind = find( obj.types==t_ind && strcmp(obj.names,name) );
    if isempty(l_ind)
        return;
    else
        value = obj.values{l_ind(end)};
        str   = obj.svalues{l_ind(end)};
    end
end
 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% display
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function display(obj)
    disp('Parameter object parameters:');
    ntitles = size( obj.titles,2);
    for t = 1:ntitles
        fprintf('TITLE   = %s \n',obj.titles{t} );
        fprintf('****************************** \n');        
        list = find(obj.tindex==t);
        for ll = list
            fprintf('name   = %s\n',obj.names{ll} );
            fprintf('type   = %s\n',obj.types{ll} );
            fprintf('string = %s\n',obj.svalues{ll} );
            fprintf('changed= %s\n\n',num2str(obj.changed(ll)) );                
        end
    end
    fprintf('\n');
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% make paramter String 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function str = mkParamString(obj)
    % make a list of all parameters (without values ...)
    
    str = {};
    ntitles = size(obj.titles,2);
    for t = 1:ntitles
        list = find(obj.tindex==t);
        for ll = list
            if isempty( obj.names{ll} )
                continue;
            end
            str = [str;obj.names{ll}];
        end
        str = [str; '------------------------------------------------'];
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% mkInfoStringShort
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function str = mkStringShort(obj)
    % create a stringlist for the top widget
    
    str     = {};
    ntitles = size(obj.titles,2);
    
    for t=1:ntitles
        % titles
        a   = linewrap(obj.titles{t},30);
        str = [ str; ['\bf\color{black}',a{1}] ;a(2:end) ];
        
        list = find(obj.tindex==t);
        for ll = list
            if isempty( obj.names{ll} )
                continue;
            end
            str = [str; ['\rm',obj.names{ll},' = ', obj.svalues{ll}]];
        end
        str = [ str;'\color{black}\rm----------------------------------------------------'];
    end
    
end


%*********************************
% mkStringLong
%*********************************
function str = mkStringLong(obj, selList)
    % create a stringlist for the right parameter widget
    
    str    = {};
    ntitles = size(obj.titles,2);
    if size(selList,1) < ntitles
        selList = [selList, ones(ntitles-size(selList,1),1)];
    end

    for t=1:ntitles
        if ~selList(t)
            continue;
        end
        a   = linewrap(obj.titles{t},30);
        str = [ str; ['\bf\color{black}',a{1}] ;a(2:end) ];
        str = [ str;'\color{black}\rm---------------------------------------------------------------'];


        list = find(obj.tindex==t);
        for ll = list
            if isempty( obj.names{ll} )
                continue;
            end
            if obj.changed(ll)
                col = '\color{red}';
            else
                col = '\color{black}';
            end
            str = [str; ['\rm',col,obj.names{ll},' = ', obj.svalues{ll}]];
        end
        str = [str;'\color{black}\rm---------------------------------------------------------------'];
    end
end



%*********************************
% checkChange
%*********************************
function obj = checkChange(obj,paramObj)
    % compare values for paramObects 
    % and set list(:,6) to changed.
    
    newlSize = size( obj.names, 2 );
    oldlSize = size( paramObj.names, 2 );
    if newlSize ~= oldlSize
        obj.changed(:) = true;
        return;
    end
    
    % leider mit Schleife ;-((
    for i=1:newlSize
        if sum((paramObj.values{i} ~= obj.values{i}))
            obj.changed(i) = true;
        else
            obj.changed(i) = false;
        end
    end
    
end

end

end
