classdef fitExp < fitClass
    %FITEXP class for lifetime fit of datapoints.
    % childclass of @fitClass
    %
    % Elmar, 14.4.2007
    methods


        %***************************
        % Init fitting stuff
        %***************************
        function obj = initFitting(obj)
            % init fit
            obj.fType    = fittype(['a*exp(-t/tau)'], ...
                'independent', 't', ...
                'dependent',   {'a','tau'} );
            obj.fOpts{1}  = fitoptions('Method','NonlinearLeastSquares');
            obj.fitName   = 'Exponential';  % internal name
            obj.minPoints = 3;              % minimum number of datapoints
        end


        %*********************************
        % init prepoints
        % calculate the prepoints
        % for the first time
        % (just from min/max of axes)
        %*********************************
        function obj = startPrePoints(obj, haxes)
            dx = xlim(haxes);
            dy = ylim(haxes);
            delx = dx(2)-dx(1);
            dely = dy(2)-dy(1);
            obj.prePoints(1,:) = [dx(1)+delx*0.2 dy(1)+dely*0.8];
            obj.prePoints(2,:) = [dx(1)+delx*0.9 dy(1)+dely*0.5];
        end


        %*********************************
        % calc the start parameters for
        % fit. Either from prePoints
        % or take them from the last fit.
        %*********************************
        function obj = getStartPoints(obj,haxes)

            if obj.adoptPoints && ~isempty(obj.fResult)
                obj.fOpts{1}.StartPoint = [obj.fResult.a,obj.fResult.tau];
            else
                % start points from prepoints
                if isempty(obj.prePoints)
                    obj = obj.startPrePoints(haxes);
                end
                tau = (obj.prePoints(1,1)-obj.prePoints(2,1))/log( abs(obj.prePoints(2,2)/obj.prePoints(1,2) ) );
                a  = obj.prePoints(1,2)/exp(-obj.prePoints(1,1)/tau);
                obj.fOpts{1}.StartPoint = [a, tau];
            end
        end

        %******************************************
        % calc fit function for given
        % prePoints (for dashed line on plot)
        %******************************************
        function [xrange,yrange] = calcDataPoints(obj,dx,prePoints)
            tau = (prePoints(1,1)-prePoints(2,1))/log( abs(prePoints(2,2)/prePoints(1,2) ) );
            a  = prePoints(1,2)/exp(-prePoints(1,1)/tau);

            xrange = linspace(dx(1),dx(2),50);
            yrange = a*exp(-xrange/tau);
        end


        %******************************************
        % calc prePoints from old fit results
        % (to set new prePoints)
        %******************************************
        function obj = last2PrePoints(obj, haxes)
            if isempty(obj.fResult)
                obj = obj.startPrePoints(haxes);
                return;
            end
            dx = xlim(haxes);
            delx = dx(2)-dx(1);
            x = [dx(1)+delx*0.2 dx(1)+delx*0.9];
            y = obj.fResult.a * exp(-x/obj.fResult.tau);
            obj.prePoints(:,1) = x;
            obj.prePoints(:,2) = y;
        end

        %***************************
        % write output to strings
        %***************************
        function obj = createOutput(obj)
            err  = confint(obj.fResult,0.68);
            max_err_a = max( err(:,1)-obj.fResult.a );
            max_err_tau = max( err(:,2)-obj.fResult.tau );

            obj.bigStr = {'\bfModel: f(x) = a*exp(-t/tau)',...
                '\rmno parameters unused'};
            obj.smallStr = sprintf('%-2.2e ms',obj.fResult.tau);

            obj.tabEntries = {'a',   sprintf('%-2.3e',obj.fResult.a)  ,sprintf('%-2.3e',max_err_a); ...
                'tau', sprintf('%-2.3e',obj.fResult.tau),sprintf('%-2.3e',max_err_tau) ...
               };
        end


    end

end
