classdef fitExpOffs < fitClass
    %FITEXP class for lifetime fit of datapoints.
    % childclass of @fitClass
    %
    % Elmar, 14.4.2007
    methods


        %***************************
        % Init fitting stuff
        %***************************
        function obj = initFitting(obj)
            % init fit
            obj.fType    = fittype(['a*exp(-t/tau)+d'], ...
                'independent', 't', ...
                'dependent',   {'a','d','tau'} );
            obj.fOpts{1}  = fitoptions('Method','NonlinearLeastSquares');
            obj.fitName   = 'Exponential';  % internal name
            obj.minPoints = 4;              % minimum number of datapoints
        end


        %*********************************
        % init prepoints
        % calculate the prepoints
        % for the first time
        % (just from min/max of axes)
        %*********************************
        function obj = startPrePoints(obj, haxes)
            dx = xlim(haxes);
            dy = ylim(haxes);
            delx = dx(2)-dx(1);
            dely = dy(2)-dy(1);
            obj.prePoints(1,:) = [dx(1)+delx*0.2 dy(1)+dely*0.8];
            obj.prePoints(2,:) = [dx(1)+delx*0.6 dy(1)+dely*0.5];
            obj.prePoints(3,:) = [dx(1)+delx*0.9 dy(1)+dely*0.01];
        end


        %*********************************
        % calc the start parameters for
        % fit. Either from prePoints
        % or take them from the last fit.
        %*********************************
        function obj = getStartPoints(obj,haxes)

            if obj.adoptPoints && ~isempty(obj.fResult)
                obj.fOpts{1}.StartPoint = [obj.fResult.a,obj.fResult.d,obj.fResult.tau];
            else
                % start points from prepoints
                if isempty(obj.prePoints)
                    obj = obj.startPrePoints(haxes);
                end
                % fix offset point
                if obj.param(1)~= 0
                    d = obj.param(1);
                else
                    d = obj.prePoints(3,2);
                    obj.fOpts{1}.Upper = [];
                    obj.fOpts{1}.Lower = [];
                end
                tau = (obj.prePoints(1,1)-obj.prePoints(2,1))/log( abs( (obj.prePoints(2,2)-d)/(obj.prePoints(1,2)-d) ) );
                a  = (obj.prePoints(1,2)-d)/exp(-obj.prePoints(1,1)/tau);
                obj.fOpts{1}.StartPoint = [a, d, tau];
                if obj.param(1)~= 0
                    d = obj.param(1);
                    obj.fOpts{1}.Upper = [max(0,2*a), max(obj.param(1)*0.9999,obj.param(1)*1.0001)];
                    obj.fOpts{1}.Lower = [min(0,1.2*a), min(obj.param(1)*0.9999,obj.param(1)*1.0001)];
                end
            end
        end

        %******************************************
        % calc fit function for given
        % prePoints (for dashed line on plot)
        %******************************************
        function [xrange,yrange] = calcDataPoints(obj,dx,prePoints)
            % fix offset point
            if obj.param(1)~= 0
                d = obj.param(1);
            else
                d = obj.prePoints(3,2);
            end
            tau = (prePoints(1,1)-prePoints(2,1))/log( abs( (prePoints(2,2)-d)/(prePoints(1,2)-d) ) );
            a  = (prePoints(1,2)-d)/exp(-prePoints(1,1)/tau);

            xrange = linspace(dx(1),dx(2),50);
            yrange = a*exp(-xrange/tau)+d;
        end


        %******************************************
        % calc prePoints from old fit results
        % (to set new prePoints)
        %******************************************
        function obj = last2PrePoints(obj, haxes)
            if isempty(obj.fResult)
                obj = obj.startPrePoints(haxes);
                return;
            end
            dx = xlim(haxes);
            delx = dx(2)-dx(1);
            x = [dx(1)+delx*0.2 dx(1)+delx*0.6 dx(1)+delx*0.9];
            y = obj.fResult.a * exp(-x/obj.fResult.tau)+obj.fResult.d;
            obj.prePoints(:,1) = x;
            obj.prePoints(:,2) = y;
            obj.prePoints(3,:) = [x(3),obj.fResult.d];
        end

        %***************************
        % write output to strings
        %***************************
        function obj = createOutput(obj)
            err  = confint(obj.fResult);
            max_err_a = max( err(:,1)-obj.fResult.a );
            max_err_tau = max( err(:,3)-obj.fResult.tau );
            max_err_d = max( err(:,2)-obj.fResult.d );

            obj.bigStr = {'\bfModel: f(x) = a*exp(-t/tau)+d',...
                '\rmfix offset by setting p1 unequal 0 '};
            obj.smallStr = sprintf('%-2.2e ms',obj.fResult.tau);

            % fix offset point
            if obj.param(1)~= 0
                obj.tabEntries = {'a',   sprintf('%-2.3e',obj.fResult.a)  ,sprintf('%-2.3e',max_err_a); ...
                'tau', sprintf('%-2.3e',obj.fResult.tau),sprintf('%-2.3e',max_err_tau); ...
                'd fixed',   sprintf('%-2.3e',obj.fResult.d)  ,'-' ...
                };
            else
                obj.tabEntries = {'a',   sprintf('%-2.3e',obj.fResult.a)  ,sprintf('%-2.3e',max_err_a); ...
                'tau', sprintf('%-2.3e',obj.fResult.tau),sprintf('%-2.3e',max_err_tau); ...
                'd',   sprintf('%-2.3e',obj.fResult.d)  ,sprintf('%-2.3e',max_err_d) ...
                };
            end
        end


    end

end
