classdef fitQuad < fitClass
%FITLINEAR class for quadratic fitting of datapoints.
% childclass of @fitClass
%
% Elmar, 2.4.2007


methods

    %***************************
    % Init fitting stuff
    %***************************
    function obj = initFitting(obj)
    
        % init fit
        obj.fType     = fittype('poly2' );   
        obj.fOpts{1}  = fitoptions('Method','NonlinearLeastSquares');
        obj.fitName   = 'quadratic';  % internal name
        obj.minPoints = 5;            % minimum number of datapoints
    
    end

 
%*********************************
% init prepoints
% calculate the prepoints
% for the first time
% (just from min/max of axes)
%*********************************
function obj = startPrePoints(obj, haxes)
    dx = xlim(haxes);
    dy = ylim(haxes);
    delx = dx(2)-dx(1);
    dely = dy(2)-dy(1);
    obj.prePoints(1,:) = [dx(1)+delx*0.1 dy(1)+dely*0.1];
    obj.prePoints(2,:) = [dx(1)+delx*0.3 dy(1)+dely*0.3];
    obj.prePoints(3,:) = [dx(1)+delx*0.6 dy(1)+dely*0.6];
end

%*********************************
% calc the start parameters for 
% fit. Either from prePoints
% or take them from the last fit.
%*********************************
function obj = getStartPoints(obj,haxes)
    
    if obj.adoptPoints && ~isempty(obj.fResult)
        obj.fOpts{1}.StartPoint = [obj.fResult.p1 obj.fResult.p2,obj.fResult.p3];
    else
        % start points from prepoints
         if isempty(obj.prePoints)
              obj = obj.startPrePoints(haxes);
         end
        % start points [p1 p2 p3]
        xx = [obj.prePoints(1,1); obj.prePoints(2,1); obj.prePoints(3,1)];
        yy = [obj.prePoints(1,2); obj.prePoints(2,2); obj.prePoints(3,2)];
        MM = [xx.*xx, xx, [1;1;1]];
        pp = inv(MM)*yy;
        obj.fOpts{1}.StartPoint = pp;
    end
end


%******************************************
% calc fit function for given
% prePoints (for dashed line on plot)
%******************************************
function [xrange,yrange] = calcDataPoints(obj,dx,prePoints)
    xx = [prePoints(1,1); prePoints(2,1); prePoints(3,1)];
    yy = [prePoints(1,2); prePoints(2,2); prePoints(3,2)];
    MM = [xx.*xx, xx, [1;1;1]];
    pp = inv(MM)*yy;
    xrange = linspace(dx(1),dx(2),50);
    yrange = pp(1)*xrange.*xrange + pp(2)*xrange + pp(3);
end

%******************************************
% calc prePoints from old fit results
% (to set new prePoints)
%******************************************
function obj = last2PrePoints(obj, haxes)
    if isempty(obj.fResult)
        obj = obj.startPrePoints(haxes);
        return;
    end
    dx = xlim(haxes);
    delx = dx(2)-dx(1);
    x  = [dx(1)+delx*0.1 dx(1)+delx*0.3 dx(1)+delx*0.6];
    y =  obj.fResult.p1 .* x.^2 + obj.fResult.p2*x+obj.fResult.p3;
    obj.prePoints(:,1) = x;
    obj.prePoints(:,2) = y;
end


%***************************
% write output to strings
%***************************
function obj = createOutput(obj)
    fSigma  = confint(obj.fResult);
    deltaP1 = max( abs(fSigma(:,1)-[obj.fResult.p1; obj.fResult.p1]) );
    deltaP2 = max( abs(fSigma(:,2)-[obj.fResult.p2; obj.fResult.p2]) );
    deltaP3 = max( abs(fSigma(:,3)-[obj.fResult.p3; obj.fResult.p3]) );

    obj.bigStr = {'\bfModel: f(x) = a*x^2 + b*x + c',...
                  '\rmno parameters unused'};
    obj.smallStr = sprintf('%-2.2e',-obj.fResult.p2/(2*obj.fResult.p1));
 
    obj.tabEntries = {'a', num2str(obj.fResult.p1), num2str(deltaP1); ...
                      'b', num2str(obj.fResult.p2), num2str(deltaP2); ...
                      'c', num2str(obj.fResult.p3), num2str(deltaP3); ...
                      'xi2',num2str(obj.gof.rsquare),''; ...
                      'MinPos',num2str(-obj.fResult.p2/(2*obj.fResult.p1)),''};
end

    
end

end
