classdef fitSawTooth < fitClass
%FITSAWTHOOTH class for sawtooth fitting of datapoints.
% childclass of @fitClass
%
% Elmar, 2.4.2007

properties
    
    %***************************
    % fit properties
    %***************************
    fitName@char       = 'SawTooth';  % internal name
    minPoints@double   = 5;           % minimum number of datapoints
    setLimits@logical  = false;
end


methods

    %***************************
    % Init fitting stuff
    %***************************
    function obj = initFitting(obj)
    
        % init fit
        obj.fType    = fittype('gauss1');  
        obj.minPoints= 5;
        obj.fType    = fittype('vsawtooth(t,tall,tup,offs,ampl,dt)',...
                               'independent', 't', ...
                               'dependent', {'ampl','dt','offs','tall','tup'} );     
        obj.doPreSetting  = true;                           
        obj.fitName   = 'SawTooth';     % internal name
        obj.fOpts{1} = fitoptions(obj.fType);
        % limits
        if obj.param(1)~= 0 && obj.param(2)~= 0
            ll = obj.param(1);
            ss = obj.param(2);
            obj.setLimits = true;
        else
            ll = 0;
            ss = 1000;
            obj.setLimits = false;            
        end
        obj.fOpts{1}.Upper = [10e8,1000,10e8,ss,1000];
        obj.fOpts{1}.Lower = [0,0,0,ll,0];
        
        
    end

    
    %***************************
    % calc start points
    %***************************
    function obj = startPrePoints(obj, haxes)
        dx = xlim(haxes);
        dy = ylim(haxes);
        delx = dx(2)-dx(1);
        dely = dy(2)-dy(1);
        
        obj.prePoints(1,:) = [dx(1)+delx*0.1 dy(1)+dely*0.1];
        obj.prePoints(2,:) = [dx(1)+delx*0.5 dy(1)+dely*0.9];
        obj.prePoints(3,:) = [dx(1)+delx*0.9 dy(1)+dely*0.1];        
    end


    %***************************
    % calc start points
    %***************************
    function obj = pre2StartPoints(obj)
        % start points [ampl offs tall tup]
        ampl = abs(obj.prePoints(2,2)-obj.prePoints(1,2));
        offs = obj.prePoints(1,2);
        tall = abs(obj.prePoints(3,1)-obj.prePoints(1,1));
        tup  = abs(obj.prePoints(2,1)-obj.prePoints(1,1));
        dt   = obj.prePoints(1,1);
        
        obj.fOpts{1}.StartPoint = [ampl dt offs tall tup];
        if obj.setLimits
            ll = obj.param(1);
            ss = obj.param(2);
            obj.fOpts{1}.Upper = [ampl*1.2, max(0,dt*1.2), offs*1.2, ss, tup*1.2];
            obj.fOpts{1}.Lower = [ampl*0.8, max(-0.001,dt*0.8), offs*0.8, ll, tup*0.8];
        else
            obj.fOpts{1}.Upper = [ampl*1.2, max(0,dt*1.2), offs*1.2, tall*1.3, tup*1.2];
            obj.fOpts{1}.Lower = [ampl*0.8, max(-0.001,dt*0.8), offs*0.8, tall*0.8, tup*0.8];
        end

    end
    
    function [xrange,yrange] = calcDataPoints(obj,dx,prePoints)

        ampl = abs(obj.prePoints(2,2)-obj.prePoints(1,2));
        offs = obj.prePoints(1,2);
        tall = abs(obj.prePoints(3,1)-obj.prePoints(1,1));
        tup = abs(obj.prePoints(2,1)-obj.prePoints(1,1));
        dt   = obj.prePoints(1,1);
        
        xrange = linspace(dx(1),dx(2),1500);           
        yrange = vsawtooth(xrange, tall, tup, offs, ampl, dt);
    end

    %***************************
    % write output to strings
    %***************************
   function obj = createOutput(obj)
        fCoeffs = coeffvalues(obj.fResult);
        fSigma  = confint(obj.fResult);

        obj.bigStr = {  ...
            [' \newline','\bf Model: f(x) = sawtooth(t,tall,tup,offs,ampl)'],' ', ...
            sprintf('  ampl = %-2.2e   (%.2e, %.2e)',fCoeffs(1),fSigma(1,1),fSigma(2,1)),...
            sprintf('  dt   = %-2.2e   (%.2e, %.2e)',fCoeffs(2),fSigma(1,2),fSigma(2,2)),...
            sprintf('  offs = %-2.2e   (%.2e, %.2e)',fCoeffs(3),fSigma(1,3),fSigma(2,3)),...
            sprintf('  tall = %-2.5e   (%.5e, %.5e)',fCoeffs(4),fSigma(1,4),fSigma(2,4)),...
            sprintf('  tup  = %-2.3e   (%.3e, %.3e)',fCoeffs(5),fSigma(1,5),fSigma(2,5)) ...            
        };
        obj.smallStr = sprintf('%-2.5e ms',fCoeffs(4));
   end
    
    % plot fit

    function obj = plotFit(obj,haxes)
        if ~isempty( obj.fResult )
            dx = xlim(haxes);
            xrange = linspace(dx(1),dx(2),3000);
            obj.handle = plot(haxes,xrange,obj.fResult(xrange), ...
                   'color',obj.plotColor,'linewidth',1.5);
        end
    end
    
end

end
