classdef fitSine < fitClass
%FITLINEAR class for simple sine
% f(x) = a*sin(w*x+p)+d
%
% childclass of @fitClass
%
% Elmar, 5.4.2007

properties
    
    %***************************
    % fit properties
    %***************************
    fitName@char       = 'Sine';  % internal name
    %fitNumber@double   = 1;         % index given by gui
    minPoints@double   = 5;         % minimum number of datapoints

end


methods

    %***************************
    % Init fitting stuff
    %***************************
    function obj = initFitting(obj)
    
        % init fit
        obj.fType    = fittype('sin1' );   
        obj.fOpts{1} = fitoptions(obj.fType);   % take first param
        
        obj.fType    = fittype('a*sin(w*(x-x0))+d', ...
                               'independent', 'x', ...
                               'dependent', {'a','d','w','x0'} );        
        obj.fOpts{1}.Upper       = [Inf,Inf,1*pi,Inf];
        obj.fOpts{1}.Lower       = [-Inf,-Inf,-1*pi,0];
        obj.fOpts{1}.Start       = [100,1000,1,1,1];
        obj.fOpts{1}.MaxFunEvals = 800;
        obj.fOpts{1}.MaxIter     = 500;
        obj.fOpts{1}.Display     = 'Off';
    
        obj.minPoints     = 5;
        obj.fitName       = 'Sine';  % internal name        
    end

    
    %***************************
    % calc start points
    %***************************
    function obj = startPrePoints(obj, haxes)
        dx = xlim(haxes);
        dy = ylim(haxes);
        obj.prePoints(1,:) = max([(dx(2)-dx(1))*0.1 (dy(2)-dy(1))*0.1],[dx(1)*1.2 dy(1)*1.2]);
        obj.prePoints(2,:) = [dx(2)*0.8 dy(2)*0.8];
    end


    %***************************
    % calc start points
    %***************************
    function obj = pre2StartPoints(obj)
        % start points [p1 p2 p3]
        a   = (obj.prePoints(2,2)-obj.prePoints(1,2))/2;
        d   = (obj.prePoints(1,2)+obj.prePoints(2,2))/2;
        lam = 2*abs(obj.prePoints(2,1)-obj.prePoints(1,1));
        w   = 6.2832/lam;
        x0  =  obj.prePoints(1,1)+lam/4;
        obj.fOpts{1}.StartPoint = [a,d,w,x0];
        bound1 = [a*2,d*1.1,w*1.5,x0*1.3];
        bound2 = [a*0.2,d*0.9,w*0.5,x0*0.7];
        obj.fOpts{1}.Upper = max(bound1,bound2);
        obj.fOpts{1}.Lower = min(bound1,bound2);
    end

    function [xrange,yrange] = calcDataPoints(obj,dx,prePoints)
        
        a   = (prePoints(2,2)-prePoints(1,2))/2;
        d   = (prePoints(1,2)+prePoints(2,2))/2;
        lam = 2*abs(prePoints(2,1)-prePoints(1,1));
        w   = 6.2832/lam;
        %p   = rem( prePoints(1,1), lam ) /lam * 6.2832;
        x0  =  obj.prePoints(1,1)+lam/4;
        xrange = linspace(dx(1),dx(2),500);           
        yrange = a*sin(w*(xrange-x0))+d;
    end

    
    %***************************
    % write output to strings
    %***************************
    function obj = createOutput(obj)
        fCoeffs = coeffvalues(obj.fResult);
        fSigma  = confint(obj.fResult);

        % prob. ms used calc freq.
        Freq = obj.fResult.w/2/pi*1000;
        obj.bigStr = {  ...
            ['\bf Model: f(x) = a  sin( w*(x-x0) ) + d'],' ' ...
              sprintf(' a  = %-2.3e   (%.3e, %.3e)\n',  fCoeffs(1),fSigma(1,1),fSigma(2,1)),...
              sprintf(' d  = %-2.3e   (%.3e, %.3e)\n',  fCoeffs(2),fSigma(1,2),fSigma(2,2)), ...
              sprintf(' w  = %-2.3e   (%.3e, %.3e)\n',  fCoeffs(3),fSigma(1,3),fSigma(2,3)), ...        
              sprintf(' x0 = %-2.3e   (%.3e, %.3e)\n',  fCoeffs(4),fSigma(1,4),fSigma(2,4)), ...                      
              sprintf(' f = %-2.3e  Hz ', Freq ) ...                
        };

        obj.smallStr = sprintf('%-2.3d Hz',Freq);
    end

    %***************************
    % plot fit
    %***************************
    function obj = plotFit(obj,haxes)
        if ~isempty( obj.fResult )
            dx = xlim(haxes);
            xrange = linspace(dx(1),dx(2),500);
            obj.handle = plot(haxes,xrange,obj.fResult(xrange), ...
                   'color',obj.plotColor,'linewidth',1.5);
        end
    end

end

end
