classdef fitSine < fitClass
%FITLINEAR class for simple sine
% f(x) = a*sin(w*x+p)+d
%
% childclass of @fitClass
%
% Elmar, 5.4.2007

methods
    
    %***************************
    % Init fitting stuff
    %***************************
    function obj = initFitting(obj)
    
        % init fit
        obj.fType    = fittype('sin1' );   
        obj.fOpts{1} = fitoptions(obj.fType);   % take first param
        
        obj.fType    = fittype('a*sin(w*(x-x0))+d', ...
                                  'independent', 'x', ...
                                  'dependent', {'a','d','w','x0'} );        
        obj.fOpts{1}.Upper       = [Inf,Inf,2*pi,Inf];
        obj.fOpts{1}.Lower       = [-Inf,-Inf,0,0];
        obj.fOpts{1}.Start       = [100,1000,1,1];
        obj.fOpts{1}.MaxFunEvals = 800;
        obj.fOpts{1}.MaxIter     = 500;
        obj.fOpts{1}.Display     = 'Off';
    
        obj.minPoints     = 5;
        obj.fitName       = 'Sine';  % internal name        
    end

    
    %***************************
    % calc start points
    %***************************
    function obj = startPrePoints(obj, haxes)
        dx = xlim(haxes);
        dy = ylim(haxes);
        deltaX = dx(2)-dx(1);
        deltaY = dy(2)-dy(1);
        obj.prePoints(1,:) = [dx(1)+deltaX*0.1 dy(1)+deltaY*0.1]; %max( [dx(1)+deltaX*0.1 dy(1)+deltaY*0.1],  [ (dx(1)*1.2 dy(1)*1.2]);
        obj.prePoints(2,:) = [dx(2)-deltaX*0.1 dy(2)-deltaY*0.1];
    end

%*********************************
% calc the start parameters for 
% fit. Either from prePoints
% or take them from the last fit.
%*********************************
function obj = getStartPoints(obj,haxes)

    if obj.adoptPoints && ~isempty(obj.fResult)
        obj.fOpts{1}.StartPoint = [obj.fResult.a obj.fResult.d,obj.fResult.w,obj.fResult.x0];
    else
        % start points from prepoints
        if isempty(obj.prePoints)
            obj = obj.startPrePoints(haxes);
        end
        a   = (obj.prePoints(2,2)-obj.prePoints(1,2))/2;
        d   = (obj.prePoints(1,2)+obj.prePoints(2,2))/2;
        lam = 2*abs(obj.prePoints(2,1)-obj.prePoints(1,1));
        w   = 6.2832/lam;
        x0  =  obj.prePoints(1,1)+lam/4;
        obj.fOpts{1}.StartPoint = [a,d,w,x0];
        bound1 = [a*2,  d*1.6, w*1.5, x0*1.3];
        bound2 = [a*0.2,d*0.2, w*0.5, x0*0.7];
        obj.fOpts{1}.Upper = max(bound1,bound2);
        obj.fOpts{1}.Lower = min(bound1,bound2);
    end
end


    function [xrange,yrange] = calcDataPoints(obj,dx,prePoints)
        
        a   = (prePoints(2,2)-prePoints(1,2))/2;
        d   = (prePoints(1,2)+prePoints(2,2))/2;
        lam = 2*abs(prePoints(2,1)-prePoints(1,1));
        w   = 6.2832/lam;
        %p   = rem( prePoints(1,1), lam ) /lam * 6.2832;
        x0  =  obj.prePoints(1,1)+lam/4;
        xrange = linspace(dx(1),dx(2),500);           
        yrange = a*sin(w*(xrange-x0))+d;
    end

    
    %******************************************
    % calc prePoints from old fit results
    % (to set new prePoints)
    %******************************************
    function obj = last2PrePoints(obj, haxes)
        if isempty(obj.fResult)
            obj = obj.startPrePoints(haxes);
            return;
        end
        %dx = xlim(haxes);
        %delx = dx(2)-dx(1);
        %x = [dx(1)+delx*0.1 dx(1)+delx*0.9];      
        x = [obj.fResult.x0+pi/(2*obj.fResult.w) obj.fResult.x0+3*pi/(2*obj.fResult.w)];
        y = obj.fResult.a*sin(obj.fResult.w*(x-obj.fResult.x0))+obj.fResult.d;
        obj.prePoints(:,1) = x;
        obj.prePoints(:,2) = y;
    end
    
    
    %***************************
    % write output to strings
    %***************************
    function obj = createOutput(obj)
        fSigma  = confint(obj.fResult,0.68);
        deltaA  = max( abs(fSigma(:,1)-[obj.fResult.a; obj.fResult.a]) );
        deltaW  = max( abs(fSigma(:,3)-[obj.fResult.w; obj.fResult.w]) );
        deltaX0 = max( abs(fSigma(:,4)-[obj.fResult.x0; obj.fResult.x0]) );
        deltaD  = max( abs(fSigma(:,2)-[obj.fResult.d; obj.fResult.d]) );
        

        % prob. ms used calc freq.
        Freq = obj.fResult.w/2/pi*1000;
        FreqSigma = Freq * deltaW/obj.fResult.w;
    
        obj.bigStr = {'\bfModel: f(x) = a  sin( w*(x-x0) ) + d',...
                      '\rmno parameters unused'};
        obj.smallStr = sprintf('%-2.2d Hz',Freq);
        obj.tabEntries = {'a', num2str(obj.fResult.a), num2str(deltaA); ...
                          'w', num2str(obj.fResult.w), num2str(deltaW); ...
                          'd', num2str(obj.fResult.d), num2str(deltaD); ...
                          'x0', num2str(obj.fResult.x0), num2str(deltaX0); ...
                          'xi2',  num2str(obj.gof.rsquare),''; ...
                          'freq', sprintf('%-2.2e',Freq),sprintf('%-2.2e',FreqSigma)};
    end

    %***************************
    % plot fit
    %***************************
    function obj = plotFit(obj,haxes)
        if ~isempty( obj.fResult )
            hold(haxes,'on');
            dx = xlim(haxes);
            xrange = linspace(dx(1),dx(2),500);
            obj.handle = plot(haxes,xrange,obj.fResult(xrange), ...
                   'color',obj.plotColor,'linewidth',1.5);
        end
    end

end

end
