classdef fitSineExp < fitClass
%FITLINEAR class for exponential damped sine
% f(x) = a*sin(w*(t-to))*exp(-t/tau) + d
%
% childclass of @fitClass
%
% Elmar, 5.4.2007

methods

    %***************************
    % Init fitting stuff
    %***************************
    function obj = initFitting(obj)
    
        % init fit
        obj.fType    = fittype('sin1' );   
        obj.fOpts{1} = fitoptions(obj.fType);   % take first param
        
        obj.fType    = fittype('a*sin(w*(t-t0))*exp(-t/tau)+d', ...
                               'independent', 't', ...
                               'dependent', {'a','d','t0','tau','w'} );        
        obj.fOpts{1}.Upper       = [Inf,Inf,Inf,Inf,1*pi];
        obj.fOpts{1}.Lower       = [-Inf,-Inf,-Inf,-Inf,-1*pi];
        obj.fOpts{1}.MaxFunEvals = 500;
        obj.fOpts{1}.MaxIter     = 300;
        obj.fOpts{1}.Display     = 'Off';

        obj.minPoints     = 6;
        obj.fitName       = 'Sin Damped';
        
    end

    
    %***************************
    % calc start points
    %***************************
    function obj = startPrePoints(obj, haxes)
        dx = xlim(haxes);
        dy = ylim(haxes);
        deltaX = dx(2)-dx(1);
        deltaY = dy(2)-dy(1);
        obj.prePoints(1,:) = [dx(1)+deltaX*0.1 dy(1)+deltaY*0.1]; %max( [dx(1)+deltaX*0.1 dy(1)+deltaY*0.1],  [ (dx(1)*1.2 dy(1)*1.2]);
        obj.prePoints(2,:) = [dx(2)-deltaX*0.4 dy(2)-deltaY*0.1];        
        obj.prePoints(3,:) = [dx(2)-deltaX*0.1 dy(2)-deltaY*0.1];    
    end


    %***************************
    % calc start points
    %***************************
    function obj = getStartPoints(obj,haxes)
        if obj.adoptPoints && ~isempty(obj.fResult)
            obj.fOpts{1}.StartPoint = [obj.fResult.a obj.fResult.d obj.fResult.t0 obj.fResult.tau obj.fResult.w];
        else
            a   = (obj.prePoints(2,2)-obj.prePoints(1,2))/2;
            d   = (obj.prePoints(1,2)+obj.prePoints(2,2))/2;
            lam = 2*abs(obj.prePoints(2,1)-obj.prePoints(1,1));
            w   = 6.2832/lam;
            t0  =  obj.prePoints(1,1)+lam/4;
            tau = -(obj.prePoints(3,1)-obj.prePoints(1,1))/log(abs(obj.prePoints(3,2)-d)/abs(a));
            obj.fOpts{1}.StartPoint = [a,d,t0,tau,w];
        
            bound1 = [a*2,d*1.1,t0*1.3,tau*1e3,w*1.5];
            bound2 = [a*0.2,d*0.9,t0*0.7,tau*1e-3,w*0.5];
            obj.fOpts{1}.Upper = max(bound1,bound2);
            obj.fOpts{1}.Lower = min(bound1,bound2);
        end
    end

    function [xrange,yrange] = calcDataPoints(obj,dx,prePoints)
        a   = (prePoints(2,2)-prePoints(1,2))/2;
        d   = (prePoints(1,2)+prePoints(2,2))/2;
        lam = 2*abs(prePoints(2,1)-prePoints(1,1));
        w   = 6.2832/lam;
        %p   = rem( prePoints(1,1), lam ) /lam * 6.2832;
        t0  =  obj.prePoints(1,1)+lam/4;
        tau = -(prePoints(3,1)-prePoints(1,1))/log(abs(prePoints(3,2)-d)/abs(a));
        xrange = linspace(dx(1),dx(2),500);           
        yrange = a*sin(w*(xrange-t0)).*exp(-xrange/tau)+d;
    end

    
    
    %******************************************
    % calc prePoints from old fit results
    % (to set new prePoints)
    %******************************************
    function obj = last2PrePoints(obj, haxes)
        if isempty(obj.fResult)
            obj = obj.startPrePoints(haxes);
            return;
        end
        %dx = xlim(haxes);
        %delx = dx(2)-dx(1);
        %x = [dx(1)+delx*0.1 dx(1)+delx*0.9];      
        x = [obj.fResult.t0-pi/(2*obj.fResult.w) obj.fResult.t0+1*pi/(2*obj.fResult.w) obj.fResult.t0+3*pi/(2*obj.fResult.w)];
        y = obj.fResult.a*sin(obj.fResult.w*(x-obj.fResult.t0)).*exp(-x./obj.fResult.tau)+obj.fResult.d;
        obj.prePoints(:,1) = x;
        obj.prePoints(:,2) = y;
    end
    

    
     %***************************
    % write output to strings
    %***************************
    function obj = createOutput(obj)
        fSigma  = confint(obj.fResult,0.68);
        deltaA  = max( abs(fSigma(:,1)-[obj.fResult.a; obj.fResult.a]) );
        deltaW  = max( abs(fSigma(:,5)-[obj.fResult.w; obj.fResult.w]) );
        deltat0 = max( abs(fSigma(:,3)-[obj.fResult.t0; obj.fResult.t0]) );
        deltatau = max( abs(fSigma(:,4)-[obj.fResult.tau; obj.fResult.tau]) );
        deltaD  = max( abs(fSigma(:,2)-[obj.fResult.d; obj.fResult.d]) );
        

        % prob. ms used calc freq.
        Freq = obj.fResult.w/2/pi*1000;
        FreqSigma = Freq * deltaW/obj.fResult.w;
    
        obj.bigStr = {'\bfModel: f(x) = a  sin( w*(x-x0) ) exp(-x/tau) + d',...
                      '\rmno parameters unused'};
        obj.smallStr = sprintf('%-2.2d Hz',Freq);
        obj.tabEntries = {'a', num2str(obj.fResult.a), num2str(deltaA); ...
                          'w', num2str(obj.fResult.w), num2str(deltaW); ...
                          'd', num2str(obj.fResult.d), num2str(deltaD); ...
                          'x0', num2str(obj.fResult.t0), num2str(deltat0); ...
                          'tau', num2str(obj.fResult.tau), num2str(deltatau); ...
                          'xi2',  num2str(obj.gof.rsquare),''; ...
                          'freq', sprintf('%-2.4e',Freq),sprintf('%-2.4e',FreqSigma)};
    end


    %***************************
    % plot fit
    %***************************
    function obj = plotFit(obj,haxes)
        if ~isempty( obj.fResult )
            hold(haxes,'on');
            dx = xlim(haxes);
            xrange = linspace(dx(1),dx(2),500);
            hold(haxes,'on');
            obj.handle = plot(haxes,xrange,obj.fResult(xrange), ...
                   'color',obj.plotColor,'linewidth',1.5);
        end
    end

end




end
