classdef fitSineExpOffs < fitClass
%FITSINEEXPOFFS class for exponential damped sine
% f(x) = a*sin(w*(t-to))*exp(-t/tau) + d
%
% childclass of @fitClass
%
% Elmar, 5.4.2007

methods

    %***************************
    % Init fitting stuff
    %***************************
    function obj = initFitting(obj)
    
        % init fit
        obj.fType    = fittype('sin1' );   
        obj.fOpts{1} = fitoptions(obj.fType);   % take first param
        
        obj.fType    = fittype('a*sin(w*(t-t0))*exp(-t/tau)+d', ...
                               'independent', 't', ...
                               'dependent', {'a','d','t0','tau','w'} );        
        obj.fOpts{1}.Upper       = [Inf,Inf,Inf,Inf,1*pi];
        obj.fOpts{1}.Lower       = [-Inf,-Inf,-Inf,-Inf,-1*pi];
        obj.fOpts{1}.MaxFunEvals = 2000;
        obj.fOpts{1}.MaxIter     = 1000;
        obj.fOpts{1}.Display     = 'Off';

        obj.minPoints     = 6;
        obj.fitName       = 'Sin Damped';
        
    end

    
    %***************************
    % calc start points
    %***************************
    function obj = startPrePoints(obj, haxes)
        dx = xlim(haxes);
        dy = ylim(haxes);
        delx = dx(2)-dx(1);
        dely = dy(2)-dy(1);
        obj.prePoints(1,:) = max([(dx(2)-dx(1))*0.1 (dy(2)-dy(1))*0.1],[dx(1)*1.2 dy(1)*1.2]);
        obj.prePoints(2,:) = [dx(1)+delx*0.5 dy(2)+dely*0.5];
        obj.prePoints(3,:) = [dx(1)+delx*0.8 dy(2)+dely*0.8];
    end

%*********************************
% calc the start parameters for 
% fit. Either from prePoints
% or take them from the last fit.
%*********************************
function obj = getStartPoints(obj,haxes)

    if obj.adoptPoints && ~isempty(obj.fResult)
        obj.fOpts{1}.StartPoint = [obj.fResult.a obj.fResult.d,obj.fResult.w,obj.fResult.x0];
    else
        % start points from prepoints
        if isempty(obj.prePoints)
            obj = obj.startPrePoints(haxes);
        end
         a   = (obj.prePoints(2,2)-obj.prePoints(1,2))/2;
        d   = (obj.prePoints(1,2)+obj.prePoints(2,2))/2;
        lam = 2*abs(obj.prePoints(2,1)-obj.prePoints(1,1));
        w   = 6.2832/lam;
        t0  =  obj.prePoints(1,1)+lam/4;
        tau = -(obj.prePoints(3,1)-obj.prePoints(1,1))/log(abs(obj.prePoints(3,2)-d)/abs(a));
        obj.fOpts{1}.StartPoint = [a,d,t0,tau,w];
        
        bound1 = [a*2,d*1.5,t0*1.3,tau*1e3,w*1.5];
        bound2 = [a*0.2,d*0.5,t0*0.7,tau*1e-3,w*0.5];
        obj.fOpts{1}.Upper = max(bound1,bound2);
        obj.fOpts{1}.Lower = min(bound1,bound2);
    end
end    

    function [xrange,yrange] = calcDataPoints(obj,dx,prePoints)
        a   = (prePoints(2,2)-prePoints(1,2))/2;
        d   = (prePoints(1,2)+prePoints(2,2))/2;
        lam = 2*abs(prePoints(2,1)-prePoints(1,1));
        w   = 6.2832/lam;
        %p   = rem( prePoints(1,1), lam ) /lam * 6.2832;
        t0  =  obj.prePoints(1,1)+lam/4;
        tau = -(prePoints(3,1)-prePoints(1,1))/log(abs(prePoints(3,2)-d)/abs(a));
        xrange = linspace(dx(1),dx(2),500);           
        yrange = a*sin(w*(xrange-t0)).*exp(-xrange/tau)+d;
    end

    
    %******************************************
    % calc prePoints from old fit results
    % (to set new prePoints)
    %******************************************
    function obj = last2PrePoints(obj, haxes)
        if isempty(obj.fResult)
            obj = obj.startPrePoints(haxes);
            return;
        end
        x = [obj.fResult.t0+pi/(2*obj.fResult.w) obj.fResult.t0+3*pi/(2*obj.fResult.w)];
        y = obj.fResult.a*sin(obj.fResult.w*(t-obj.fResult.t0)).*exp(-t/obj.fResult.tau) +obj.fResult.d;
        obj.prePoints(:,1) = x;
        obj.prePoints(:,2) = y;
    end
    
    
    %***************************
    % write output to strings
    %***************************
    function obj = createOutput(obj)
        fCoeffs = coeffvalues(obj.fResult);
        fSigma  = confint(obj.fResult,0.68);
        
        Freq = obj.fResult.w/2/pi*1000;
        obj.bigStr = { ' ' ...
            ['\bf Model: f(x) = a exp(-t/\tau) sin( w*(t-t_0) ) + d'],' ' ...
              sprintf(' a    = %-2.3e   (%.3e, %.3e)',  fCoeffs(1),fSigma(1,1),fSigma(2,1)),...
              sprintf(' d    = %-2.3e   (%.3e, %.3e)',  fCoeffs(2),fSigma(1,2),fSigma(2,2)), ...
              sprintf(' t0   = %-2.3e   (%.3e, %.3e)',  fCoeffs(3),fSigma(1,3),fSigma(2,3)), ...                      
              sprintf(' tau = %-2.3e   (%.3e, %.3e)',  fCoeffs(4),fSigma(1,4),fSigma(2,4)), ...                      
              sprintf(' w    = %-2.4e   (%.4e, %.4e)\n',  fCoeffs(5),fSigma(1,5),fSigma(2,5)), ...        
              sprintf(' f    = %-2.4e  Hz ', Freq ) ...                
        };

        obj.smallStr = sprintf('%-2.4d Hz',Freq);
    end

    %***************************
    % plot fit
    %***************************
    function obj = plotFit(obj,haxes)
        if ~isempty( obj.fResult )
            hold(haxes,'on');            
            dx = xlim(haxes);
            xrange = linspace(dx(1),dx(2),500);
            obj.handle = plot(haxes,xrange,obj.fResult(xrange), ...
                   'color',obj.plotColor,'linewidth',1.5);
        end
    end

end




end
