classdef fitTemp < fitClass
    %FITTEMP class for temperature fit of datapoints.
    % childclass of @fitClass
    %
    % Elmar, 2.4.2007

    methods

        %***************************
        % Init fitting stuff
        %***************************
        function obj = initFitting(obj)
            % init fit
            obj.fType     = fittype(['sqrt( sig0^2 + (sigv * (t+',num2str(obj.param(1)),'))^2 )'], ...
                'independent', 't', ...
                'dependent',  {'sig0','sigv'} );
            obj.fOpts{1}  = fitoptions('Method','NonlinearLeastSquares');
            obj.fOpts{1}.Lower = [0,0];
            obj.fOpts{1}.Upper = [Inf,Inf];
            obj.fitName   = 'Temperature';  % internal name
            obj.minPoints = 4;              % minimum number of datapoints
        end


        %*********************************
        % init prepoints
        % calculate the prepoints
        % for the first time
        % (just from min/max of axes)
        %*********************************
        function obj = startPrePoints(obj, haxes)
            dx = xlim(haxes);
            dy = ylim(haxes);
            delx = dx(2)-dx(1);
            dely = dy(2)-dy(1);
            obj.prePoints(1,:) = [dx(1)+delx*0.2 dy(1)+dely*0.2];
            obj.prePoints(2,:) = [dx(1)+delx*0.6 dy(1)+dely*0.6];
        end


        %*********************************
        % calc the start parameters for
        % fit. Either from prePoints
        % or take them from the last fit.
        %*********************************
        function obj = getStartPoints(obj,haxes)

            if obj.adoptPoints && ~isempty(obj.fResult)
                obj.fOpts{1}.StartPoint = [obj.fResult.sig0 obj.fResult.sigv];
            else
                % start points from prepoints
                if isempty(obj.prePoints)
                    obj = obj.startPrePoints(haxes);
                end
                % start points [p1 p2 p3]
                yy = [obj.prePoints(1,2)^2; obj.prePoints(2,2)^2];
                MM = [1 (obj.prePoints(1,1)+obj.param(1))^2; 1 (obj.prePoints(2,1)+obj.param(1))^2];
                pp = inv(MM)*yy;
                obj.fOpts{1}.StartPoint = sqrt(pp);
            end
        end

        %******************************************
        % calc fit function for given
        % prePoints (for dashed line on plot)
        %******************************************
        function [xrange,yrange] = calcDataPoints(obj,dx,prePoints)
            yy = [prePoints(1,2)^2; prePoints(2,2)^2];
            MM = [1 (prePoints(1,1)+obj.param(1))^2; 1 (prePoints(2,1)+obj.param(1))^2];
            pp = inv(MM)*yy;
            xrange = linspace(dx(1),dx(2),250);
            yrange = sqrt( pp(1)+ pp(2).*(xrange+obj.param(1)).^2);
        end


        %******************************************
        % calc prePoints from old fit results
        % (to set new prePoints)
        %******************************************
        function obj = last2PrePoints(obj, haxes)
            if isempty(obj.fResult)
                obj = obj.startPrePoints(haxes);
                return;
            end
            dx   = xlim(haxes);
            delx = dx(2)-dx(1);
            x    = [dx(1)+delx*0.2 dx(1)+delx*0.6];
            y    = sqrt(obj.fResult.sig0^2 + (obj.fResult.sigv*(x+obj.param(1))).^2);

            obj.prePoints(:,1) = x;
            obj.prePoints(:,2) = y;
        end



        %***************************
        % write output to strings
        %***************************
        function obj = createOutput(obj)
            err       = confint(obj.fResult,0.68);
            max_err_v = max( err(:,2)-obj.fResult.sigv );
            max_err_0 = max( err(:,1)-obj.fResult.sig0 );

            Temp       = 0.016 * obj.fResult.sigv*obj.fResult.sigv;
            Temp_err   = 0.016 * obj.fResult.sigv*max_err_v;

            obj.bigStr = {['\bfModel: \sigma_t = \surd ( \sigma_0^2 + (\sigma_v^2 * (t+',num2str(obj.param(1)),')^2 ))'],...
                '\rm  used parameter p1 to shift time'};
            obj.smallStr = sprintf('%-2.2e uK',Temp);

            obj.tabEntries = {'sig0', num2str(obj.fResult.sig0), num2str(max_err_0); ...
                'sigv', num2str(obj.fResult.sigv), num2str(max_err_v); ...
                'xi2'  , num2str(obj.gof.rsquare),''; ...
                'Temp' , sprintf('%-2.3e uK',Temp),sprintf('%-2.3e uK',Temp_err)};
        end

    end

end
