function [G,Xbar] = mgroup(X, FUNC)
%
% function [Xbar] = group (X)
% function [Xbar] = group (X, 'FUNC')
% function [G,Xbar] = group (X)
% function [G,Xbar] = group (X, 'FUNC')
%
% Returns the summary of the columns of X grouped by the first
% column of X. 'FUNC' is the summary function.  If 'FUNC' is omitted, 
% 'mean' is used.  'length' can be used to give a count of data
% by group.
%
% If X contains more than 2 columns, each additional row of 'FUNC'
% may contain a function to use to group each each additional column.
%
% If only one function is given, it is used to summarize all columns of X.
% If the function names are not the same length, pad the strings with
% trailing spaces. 
%
% With one output argument, the summaries are returned in a table,
% [G Xbar].  With two output arguments, the group vector and table of
% summaries are returned as two variables, G and Xbar.
%
% Example:  Summarize a set of data X grouped by measurement time TIME.
%           The returned grouped table has columns: TIME, MEAN, STD, N
%
%       group ([TIME X X X], ['mean  '; 'std   '; 'length']);
%


%----------------------------------------------------------------------
% Copyright (c) 1997.  Don R. Maszle.  All rights reserved.
%
%   -- Revisions -----
%        Date:  1 March 1997
%      Author:  Don R. Maszle
%      E-mail:  maze@sparky.berkeley.edu
%   -- SCCS  ---------
%----------------------------------------------------------------------



%-- DIAGNOSTICS ----------


if (nargin < 1 | nargin > 2 | nargout > 2)
  help group
  return
end
G = [];



if (nargin == 1)
  FUNC = 'mean';
end


%-- SETUP ----------


%-- The number of columns of data to group
cCols = size(X,2)-1;


%-- Allow one function to be applied to each column of matrix
if (cCols > 1 & size(FUNC,1) == 1)
  FUNC = setstr(ones(cCols,1)*FUNC);
end


if (size(FUNC,1) ~= cCols)
  disp('group:  Number of grouping functions in FUNC must equal number');
  disp('        of data columns in X.');
  return;
end


%-- Get the unique keys and report the groupings sorted on the key


keys = sort(unique(X(:,1)));
cKeys = size(keys,1);
G = [keys keys*zeros(1,cCols)];


for j = 2:cCols+1
  for i = 1:cKeys
    xi = X(find(X(:,1) == keys(i)),:);
    G(i,j) = feval(deblank(FUNC(j-1,:)), xi(:,j));
  end
end


if (nargout == 2)
  Xbar = G(:,2:end);
  G = G(:,1);
end